<?php
	require_once(CI_PATH_LIBS.'/config.inc.php');
	require_once(CI_PATH_LIBS.'/auth.inc.php');
	require_once(CI_PATH_LIBS.'/users.inc.php');
	require_once(CI_PATH_LIBS.'/groups.inc.php');
	require_once(CI_PATH_LIBS.'/sql.inc.php');
	
	class CI_ADDRESSBOOK
	{
		private $level;
		
		function CI_ADDRESSBOOK($level='public')
		{
			$levels=$this->get_folders();
			if (!$levels[$level]) $level='public';
			
			$this->level=$level;
		}

		function get_map($info)
		{
			return array(
				'destination'			=> $info['destination'],
				'ud_castiron_name'		=> $info['userdata']['castiron-name'],
				'ud_castiron_surname'	=> $info['userdata']['castiron-surname'],
				'ud_castiron_company'	=> $info['userdata']['castiron-company'],
				'ud_castiron_email'		=> $info['userdata']['castiron-email'],
				'ud_castiron_mobile'	=> $info['userdata']['castiron-mobile'],
				'ud_castiron_office'	=> $info['userdata']['castiron-office'],
				'ud_castiron_extension'	=> $info['userdata']['castiron-extension'],
				'ud_castiron_home'		=> $info['userdata']['castiron-home'],
				'ud_castiron_url'		=> $info['userdata']['castiron-url']
			);
		}

		function get_rmap($info)
		{
			return array(
				'destination'				=> $info['destination'],
				'userdata'	=> array(
					'castiron-name'			=> 	$info['ud_castiron_name'],
					'castiron-surname'		=>	$info['ud_castiron_surname'],
					'castiron-company'		=>	$info['ud_castiron_company'],
					'castiron-email'		=>	$info['ud_castiron_email'],
					'castiron-mobile'		=>	$info['ud_castiron_mobile'],
					'castiron-office'		=>	$info['ud_castiron_office'],
					'castiron-extension'	=>	$info['ud_castiron_extension'],
					'castiron-home'			=>	$info['ud_castiron_home'],
					'castiron-url'			=>	$info['ud_castiron_url']
				)
			);
		}
		
		function get_config_path()
		{
			return IGX_PATH_BASE.'/config/maps';
		}

		function get_config_file($folder)
		{
			list($level, $arg)=split(':', $folder);

			if ($level == 'user') return $this->get_config_path().'/user-'.$arg.'.xml';
			if ($level == 'group') return $this->get_config_path().'/group-'.$arg.'.xml';

			return $this->get_config_path().'/public.xml';
		}

		function get_folder_name($level, $arg='')
		{
			if ($level == 'user') return 'user:'.$arg;
			if ($level == 'group') return 'group:'.$arg;
			return 'public';
		}

		function get_folders()
		{
			$out=array();
			$info=CI_AUTH::get_login_info();

			$manager_users	= new CI_USERS();
			$manager_groups	= new CI_GROUPS();

			$out['public']=ci_lbl('libaddressbook.level.public');

			if ($info['ud_castiron_role'] == 'admin')
			{
				$groups	= $manager_groups->get_list();
				$users	= $manager_users->get_list();

				foreach ($groups as $info2)
					$out['group:'.$info2['id']]=ci_lbl('libaddressbook.level.group', array('id' => $info2['id']));

				foreach ($users as $info2)
				{
					if (($info['id'] != 'admin') && ($info2['ud_castiron_role'] == 'admin') && ($info['id'] != $info2['id'])) continue;
					$out['user:'.$info2['id']]=ci_lbl('libaddressbook.level.user', array('id' => $info2['id']));
				}
			}
			
			elseif ($info['ud_castiron_role'] == 'groupadmin')
			{
				$users=$manager_users->get_list();

				$out['group:'.$info['group']]=ci_lbl('libaddressbook.level.mygroup');

				foreach ($users as $info2)
				{
					if ($info2['group'] != $info['group']) continue;
					if (($info2['ud_castiron_role'] == 'groupadmin') && ($info['id'] != $info2['id'])) continue;
					$out['user:'.$info2['id']]=ci_lbl('libaddressbook.level.user', array('id' => $info2['id']));
				}
			}

			else
			{
				$out['group:'.$info['group']]=ci_lbl('libaddressbook.level.mygroup');
				$out['user:'.$info['id']]=ci_lbl('libaddressbook.level.myuser');
			}

			return $out;
		}

		function get_list()
		{
			$list			= array();
			$list_public	= array();
			$list_group		= array();
			$list_user		= array();

			list($level, $arg)=split(':', $this->level);

			/* Read public addressbook */
			$list_public = @ci_get_unserialized_xml($this->get_config_file($this->get_folder_name('public')));

			/* Read group addressbook */
			if ($level == 'group')
			{
				$list_group = @ci_get_unserialized_xml($this->get_config_file($this->get_folder_name('group', $arg)));
				$user=$arg;
			}

			/* Read group + user addressbook */
			elseif ($level == 'user')
			{
				$manager=new CI_USERS();
				$info=$manager->get_info($arg);
			
				$list_group	= @ci_get_unserialized_xml($this->get_config_file($this->get_folder_name('group', $info['group'])));
				$list_user	= @ci_get_unserialized_xml($this->get_config_file($this->get_folder_name('user', $arg)));

				$user=$arg;
				$group=$info['group'];
			}

			/* merge the addressbooks */
			$list =& $list_public;

			if (is_array($list_group) && is_array($list_group['items']))
				foreach ($list_group['items'] as $k => $v)
				{
					$list_group['items'][$k]['folder']=$this->get_folder_name('group', $group);
					$list['items'][$k] =& $list_group['items'][$k];
				}

			if (is_array($list_user) && is_array($list_user['items']))
				foreach ($list_user['items'] as $k => $v)
				{
					$list_user['items'][$k]['folder']=$this->get_folder_name('user', $user);
					$list['items'][$k] =& $list_user['items'][$k];
				}

			/* Beautify... */
			if (!$list['items']) $list['items']=array();
			foreach ($list['items'] as $k => $v)
			{
				$short=preg_replace("/^_/", '', $k);
				$list['items'][$short]=$v;

				unset($list['items'][$k]);

				$folder=$list['items'][$short]['folder'];
				$list['items'][$short]=$this->get_map($list['items'][$short]);
				$list['items'][$short]['folder']=$folder;

				if (!$list['items'][$short]['folder']) $list['items'][$short]['folder']='public';
			}

			return $list['items'];
		}

		function get_info($id)
		{
			$list =& $this->get_list();
			$list[$id]['shortcut']=$id;
			return $list[$id];
		}

		function shortcut_exists($s)
		{
			$list =& $this->get_list();
			if ($list[$s]) return true;

			return false;
		}

		function get_search($q)
		{
			$list =& $this->get_list();

			if ($q)
				foreach ($list as $short => $info)
				{
					if ($short == $q) continue;
					if (strpos($info['destination'], $q)			!== false) continue;
					if (strpos($info['ud_castiron_name'], $q)		!== false) continue;
					if (strpos($info['ud_castiron_email'], $q) 		!== false) continue;
					if (strpos($info['ud_castiron_surname'], $q)	!== false) continue;
					if (strpos($info['ud_castiron_company'], $q)	!== false) continue;

					unset($list[$short]);
				}

			ksort($list);

			return $list;
		}

		function task_delete($id)
		{
			list($folder, $short)=split('/', $id);

			$file=$this->get_config_file($folder);
			if (!$file) return;

			$list =& ci_get_unserialized_xml($file);

			/* Remove the previous */
			unset($list['items']['_'.$short]);
			$this->save_raw_config($list, $folder);
			
			return ci_response_ok();
		}

		function task_save($info)
		{
			list($folder, $short) = split('/', $info['id']);

			$file=$this->get_config_file($folder);
			if (!$file) return;

			$list =& ci_get_unserialized_xml($file);
			if (PEAR::isError($list)) return ci_response_error(
				'unknown',
				array(
					'desc'		=> ci_lbl('libaddressbook.error.cannot-read')
				)
			);

			$fields =& $this->get_rmap($info);
			$list['items']['_'.$short] =& $fields;

			$this->save_raw_config($list, $folder);

			/* Remove the previous */
			if ($info['old_id'] && ($info['old_id'] != $info['id']))
				$this->task_delete($info['old_id']);
			
			return ci_response_ok();
		}

		function save_raw_config($array, $folder='public')
		{
			ci_save_xml($this->get_config_file($folder), $array, 'map');
			return ci_response_ok();
		}
	}
?>