<?php
	require_once(CI_PATH_PEAR.'/XML/Serializer.php');
	require_once(CI_PATH_PEAR.'/XML/Unserializer.php');
	
/* XML serializer */
	function ci_get_serialized_xml($array, $root='config')
	{
		$serializer_options = array (
 			"indent"			=> "\t",
 			"linebreak"			=> "\n",
 			"addDecl"			=> true,
			"encoding"			=> "utf-8",
			"attributesArray"	=> '@',
			"contentName"		=> '#',
			"mode"				=> 'simplexml',
			'rootName' 			=> $root
		);

		$serializer = &new XML_Serializer($serializer_options);
		$serializer->serialize($array);
		
		return $serializer->getSerializedData();
	}

/* XML saver */
	function ci_save_xml($file, $array, $root='config')
	{
		$fp=fopen($file, "w");
		fputs($fp, ci_get_serialized_xml($array, $root));
		fclose($fp);
	}

/* XML unserializer */
	function ci_get_unserialized_xml($xml)
	{
		if (!$xml) return '';

		if (is_file($xml)) $xml=file_get_contents($xml);

		$serializer_options = array (
			'contentName'	=> '#',
			'parseAttributes'	=> true,
			'attributesArray'	=> '@'
			
		);

		$unserializer = & new XML_Unserializer($serializer_options); 
		$unserializer->unserialize($xml);
		
		return $unserializer->getUnserializedData();
	}

/* Label */
	function ci_lbl($lbl_id, $vars=array())
	{
		global $CI_LBL;

		if (is_null($CI_LBL[$lbl_id])) return $lbl_id;

		$out=$CI_LBL[$lbl_id];
		preg_match_all('/\$\{(?P<var>\w+)\}/', $out, $matches);

		foreach ($matches['var'] as $k) $out=str_replace('${'.$k.'}', $vars[$k], $out);

		return $out;
	}

/* Helps */
	function ci_hlp($lbl_id, $vars=array())
	{
		global $CI_LBL;
		global $CI_CORE_CONFIG;

		$vars['sysname']=ucfirst($CI_CORE_CONFIG['hq']['sysname']);

		$text=@file_get_contents(CI_PATH_LANGS.'/'.$CI_CORE_CONFIG['language'].'/'.$lbl_id.'.hlp.html');

		if (!$text) return '';
		preg_match_all('/\$\{(?P<var>\w+)\}/', $text, $matches);

		foreach ($matches['var'] as $k) $text=str_replace('${'.$k.'}', $vars[$k], $text);

		return $text;
	}
	
/* XML response OK */
	function ci_response_ok($params=array())
	{
		return array(
			"type"		=>	"ok",
			"params"	=> $params
		);
	}

/* XML response ERROR */
	function ci_response_error($reason="unknown", $params=array())
	{
		return array(
			"type"		=>	"error",
			"reason"	=> $reason,
			"params"	=> $params
		);
	}

/* Normalize an XML array */
	function ci_normalize_xml_array($a)
	{
		if (!sizeof($a)) return array();
		if (!is_array($a)) return array($a);

		if (!$a[0]) return array($a);
		return $a;
	}

/* Retrieves the OS time zones */
	function ci_get_os_timezones()
	{
		$out=array();

		$lines=file("/usr/share/zoneinfo/zone.tab");
		foreach ($lines as $line)
		{
			$line=preg_replace("/[\n\r]+/", '', $line);
			if (preg_match("/^#/", $line)) continue;

			$fields=preg_split("/\t/", $line);
			array_push($out, $fields[2]);
		}

		sort($out);
		return $out;
	}

/* Versione evaluation */
	function ci_license_is_evaluation()
	{
		global $CI_CORE_CONFIG_LICENSE;
		if (!$CI_CORE_CONFIG_LICENSE) return true;

		if (!strlen($CI_CORE_CONFIG_LICENSE['serial'])) return true;
		if ($CI_CORE_CONFIG_LICENSE['mc'] != ci_license_get_mc()) return true;

		return false;
	}

	function ci_get_version()
	{
		return trim(shell_exec(CORE_PATH_BASE."/libexec/version"));
	}

	function ci_license_get_mc()
	{
		return trim(shell_exec(CORE_PATH_BASE."/libexec/machinecode"));
	}

/* Retrieves the known languages list */
	function ci_get_languages()
	{
		$out=array();

		$lines=file(CI_PATH_MISC.'/languages.txt');
		foreach ($lines as $line)
		{
			list($code, $desc)=preg_split("/\t+/", $line, 2);
			$out[strtoupper($code)]=trim($desc);
		}

		ksort($out);
		return $out;
	}

/* Retrieves the known countries list */
	function ci_get_countries()
	{
		$out=array();

		$lines=file(CI_PATH_MISC.'/countries.txt');
		foreach ($lines as $line)
		{
			list($code, $desc)=preg_split("/\t+/", $line, 2);
			$out[strtoupper($code)]=trim($desc);
		}
		
		return $out;
	}
	
/* Get the current time zone */
	function ci_get_os_timezone()
	{
		return preg_replace("/^\/usr\/share\/zoneinfo\//", "", @readlink("/etc/localtime"));
	}

/* Retrives the CI languages */
	function ci_get_ui_languages()
	{
		global $CI_CONFIG_SYSTEM;
		
		$out=array();
		$dh=opendir(CI_PATH_LANGS);

		while($f=readdir($dh))
		{
			if (!is_dir(CI_PATH_LANGS.'/'.$f)) continue;
			if ($f[0] == '.') continue;

			array_push($out, $f);
		}

		return $out;
	}

/* Sets a core flag */
	function ci_set_core_flag($name, $value='')
	{
		$name=preg_replace("/[^\w\.\-\_]+/", '', $name);
		$name=preg_replace("/^\.+/", '', $name);
		if (!$name) return;
		
		return file_put_contents(CORE_PATH_BASE."/flags/$name.flag", $value);
	}

/* Gets a core flag */
	function ci_get_core_flag($name)
	{
		$name=preg_replace("/[^\w\.\-\_]+/", '', $name);
		$name=preg_replace("/^\.+/", '', $name);
		if (!$name) return;
		
		return @file_get_contents(CORE_PATH_BASE."/flags/$name.flag");
	}


/* Text crop */
	function ci_crop_string($s, $len, $ext='...')
	{
		if (strlen($s)>$len)
			return substr($s, 0, $len-strlen($ext)).$ext;
		
		return $s;
	}

/* Restituisce un dato di durata leggibile */
	function ci_readable_duration($i, $extended=false)
	{
		$d=intval($i/86400);
		$i=$i-$d*86400;
		
		$h=intval($i/3600);
		$i=$i-$h*3600;

		$m=intval($i/60);
		$i=intval($i-$m*60);

		if ($extended)
		{
			$out=array();
			if ($d) array_push($out, "$d day(s)");
			if ($h) array_push($out, "$h hour(s)");
			if ($m) array_push($out, "$m minute(s)");
			
			array_push($out, "$i second(s)");

			return join(', ', $out);
		}

		$out='';

		if ($d) $out="$d".'d, ';
		$out.=str_pad($h, 2, '0', STR_PAD_LEFT).':'.str_pad($m, 2, '0', STR_PAD_LEFT).':'.str_pad($i, 2, '0', STR_PAD_LEFT);

		return $out;
	}

/* Restituisce un dato numerico leggibile */
	function ci_readable_num($n)
	{
		if ($n<1000)
		{
			$mul='';
		}
		elseif ($n<1000000)
		{
			$mul='K';
			$n=$n/1000;
		}
		elseif ($n<1000000000)
		{
			$mul='M';
			$n=$n/1000/1000;
		}
		elseif ($n<1000000000000)
		{
			$mul='G';
			$n=$n/1000/1000/1000;
		}

		return number_format($n, 2, '.', '').' '.$mul;
	}
	
/* Restituisce un dato di dimensione file leggibile */
	function ci_readable_size($n)
	{
		if ($n<1000)
		{
			$mul='B';
		}
		elseif ($n<1000000)
		{
			$mul='KB';
			$n=$n/1000;
		}
		elseif ($n<1000000000)
		{
			$mul='MB';
			$n=$n/1000/1000;
		}
		elseif ($n<1000000000000)
		{
			$mul='GB';
			$n=$n/1000/1000/1000;
		}

		return number_format($n, 2, '.', '').' '.$mul;
	}
?>