#!/bin/bash

cd `dirname $0`
MYDIR=`pwd`
MYSCRIPT=`basename $0`
cd ../..
S2P_PATH_BASE=`pwd`
cd $MYDIR

S2P_PATH_RUN=`$S2P_PATH_BASE/core/libexec/config_view.py system simple | grep ^path.run | cut -d= -f2`
S2P_OS_USERNAME=`$S2P_PATH_BASE/core/libexec/config_view.py system simple | grep ^os.username | cut -d= -f2`
if [[ `whoami` != "$S2P_OS_USERNAME" ]]; then
	exec sudo -u $S2P_OS_USERNAME ./$MYSCRIPT $@
fi

usage()
{
	echo "Usage: $0 username start|restart|align|profiles|stop"
	cd $OWD

	exit -1
}

lock_pid()
{
	if [ -f $S2P_PATH_RUN/sessions.sh.pid ]; then
		echo "$S2P_PATH_RUN/sessions.sh.pid already exists"
		echo "Is another instance running?"
		exit -1
	fi

	echo $$ > $S2P_PATH_RUN/sessions.sh.pid
}

unlock_pid()
{
	rm -f $S2P_PATH_RUN/sessions.sh.pid
}

if [ -z "$1" ]; then
	usage
fi

if [ "$1" == "restart" ]; then
	./$MYSCRIPT stop $2
	./$MYSCRIPT start $2
	exit 0
fi

SESSIONS_PER_TIME=10
SESSIONS_INTERVAL=1

SESSIONS_RUNNING=`$S2P_PATH_BASE/core/libexec/list_running.py`
SESSIONS_CONFIGURED=`$S2P_PATH_BASE/core/libexec/list_configured.py`

SESSIONS_C="0"

lock_pid

if [ "$1" == "start" ]; then

	# Stop running sessions
	for SESSION in $SESSIONS_RUNNING; do
		$S2P_PATH_BASE/core/bin/session.sh $SESSION stop $2
	done

	# Start configured sessions
	for SESSION in $SESSIONS_CONFIGURED; do
		SESSIONS_C=`expr $SESSIONS_C + 1`

		$S2P_PATH_BASE/core/bin/session.sh $SESSION start $2

		if [[ "$SESSIONS_C" -ge "$SESSIONS_PER_TIME" ]]; then
			sleep $SESSIONS_INTERVAL
			SESSIONS_C="0"
		fi
	done

elif [ "$1" == "align" ]; then

	# Locate sessions to stop
	for SESSION in $SESSIONS_RUNNING; do
		IS_OK=`echo "$SESSIONS_CONFIGURED" |grep ^$SESSION$ -c`

		if [ "$IS_OK" -lt "1" ]; then
			$S2P_PATH_BASE/core/bin/session.sh $SESSION stop $2
		fi
	done

	# Locate sessions to start
	for SESSION in $SESSIONS_CONFIGURED; do
		SESSIONS_C=`expr $SESSIONS_C + 1`

		IS_RUNNING=`echo "$SESSIONS_RUNNING" |grep ^$SESSION$ -c`

		if [ "$IS_RUNNING" -lt "1" ]; then
			$S2P_PATH_BASE/core/bin/session.sh $SESSION start $2
		fi

		if [[ "$SESSIONS_C" -ge "$SESSIONS_PER_TIME" ]]; then
			sleep $SESSIONS_INTERVAL
			SESSIONS_C="0"
		fi
	done

elif [ "$1" == "profiles" ]; then

	rm -f $S2P_PATH_BASE/core/flags/sessions.reloadprofiles.flag &> /dev/null
	$S2P_PATH_BASE/core/libexec/reload_profiles.py

else

	# Stop running sessions
	for SESSION in $SESSIONS_RUNNING; do
		$S2P_PATH_BASE/core/bin/session.sh $SESSION stop $2
	done
	
fi

unlock_pid

exit 0
