#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT


########################################################################################
# Calls management

class S2P_SKYPE_CALL:

	def __init__(self, parent, source, callid):
		self._callid=callid
		self._parent=parent
		self._properties={}
		self._source=source
		self._status="undefined"

	def __repr__(self):
		return "[s2p-skype-call: %s/%s]" % (self._source, self._callid)

	def update(self, properties):
		self._properties=properties
		self._properties['status']=self._properties['status'].lower()
		if self._properties['status']: self._status=self._properties['status']

	def _command(self, command, args={}):
		args['call-id']=self._callid

		return self._parent.command(self._source, command, args)

########################################################################################
# Public functions for info

	def get_id(self):
		return self._callid

	def get(self, key):
		if not self._properties.has_key(key): return ""
		return self._properties[key]

	def get_source(self):
		return self._source

	def get_status(self):
		return self._status

########################################################################################
# Public functions for calls

	def set_tcp_capture(self, port):
		res=self._command("call-set-input-port", {'port': str(port)})
		return (not self._parent.is_error(res), res)

	def set_tcp_playback(self, port):
		res=self._command("call-set-output-port", {'port': str(port)})
		return (not self._parent.is_error(res), res)
	
	def send_finished(self, args={}):
		res=self._command("call-hangup", args)
		return (not self._parent.is_error(res), res)

	def send_forward(self, args={}):
		res=self._command("call-forward", args)
		return (not self._parent.is_error(res), res)

	def send_answer(self, args={}):
		res=self._command("call-answer", args)
		return (not self._parent.is_error(res), res)

	def send_hold(self, args={}):
		res=self._command("call-hold", args)
		return (not self._parent.is_error(res), res)

	def send_resume(self, args={}):
		res=self._command("call-resume", args)
		return (not self._parent.is_error(res), res)

	def send_dtmf(self, args={}):
		res=self._command("call-dtmf", args)
		return (not self._parent.is_error(res), res)
	
